@@ -6,7 +6,7 @@
 
 package Convert::ASN1::parser;
 {
-  $Convert::ASN1::parser::VERSION = '0.24';
+  $Convert::ASN1::parser::VERSION = '0.25';
 }
 
 use strict;
@@ -916,11 +916,19 @@ sub yylex {
     next if defined $1; # comment or whitespace
 
     if (defined $2 or defined $3) {
+      my $ret = $+;
+
       # A comma is not required after a '}' so to aid the
       # parser we insert a fake token after any '}'
-      push @stacked, constPOSTRBRACE() if defined $2 and $+ eq '}';
+      if ($ret eq '}') {
+        my $p   = pos($asn);
+        my @tmp = @stacked;
+        @stacked = ();
+        pos($asn) = $p if yylex() != constCOMMA();    # swallow it
+        @stacked = (@tmp, constPOSTRBRACE());
+      }
 
-      return $reserved{$yylval = $+};
+      return $reserved{$yylval = $ret};
     }
 
     if (defined $4) {
